@@ -1,6 +1,6 @@
 package Git::More;
 {
-  $Git::More::VERSION = '0.051';
+  $Git::More::VERSION = '0.052';
 }
 # ABSTRACT: A Git extension with some goodies for hook developers.
 
@@ -21,99 +21,6 @@ use Carp;
 use File::Slurp;
 use Git::Hooks qw/:utils/;
 
-sub _compatibilize_config {
-    my ($config) = @_;
-
-    # Up to version 0.022 the plugins used flat names, such as
-    # "check-acls.pl". These names were used as values for the
-    # githooks.HOOK configuration variables and also as the name of
-    # configuration sections specific of the plugins. In version 0.023
-    # the three existing plugins (check-acls.pl, check-jira.pl, and
-    # check-structure.pl) were converted to proper modules and renamed
-    # to the usual CamelCase form of the names (i.e., CheckAcls.pm,
-    # CheckJira.pm, and CheckStructure.pm). To preserve compatibility
-    # with already configured hooks here we inject the old names in
-    # the new names.
-
-    foreach my $hook (qw/commit-msg pre-commit pre-receive post-receive update/) {
-        if (exists $config->{githooks}{$hook}) {
-            foreach (@{$config->{githooks}{$hook}}) {
-                $_ = "Check\u$1" if /^check-(acls|jira|structure)(?:\.pl)?$/;
-            }
-        }
-    }
-
-    foreach my $name (
-        ['check-acls'      => 'checkacls'],
-        ['check-jira'      => 'checkjira'],
-        ['check-structure' => 'checkstructure'],
-    ) {
-        if (exists $config->{$name->[0]}) {
-            if (exists $config->{$name->[1]}) {
-                die  __PACKAGE__, ": you have incompatible configuration sections: '$name->[0]' and '$name->[1]'.\n",
-                    "Please, rename all variables from section '$name->[0]' to section '$name->[1]'.\n";
-            } else {
-                $config->{$name->[1]} = delete $config->{$name->[0]};
-            }
-        }
-    }
-
-    # Up to version 0.020 the configuration variables 'admin' and
-    # 'userenv' were defined for the CheckAcls and CheckJira
-    # plugins. In version 0.021 they were both "promoted" to the
-    # Git::Hooks module, so that they can be used by any access
-    # control plugin. In order to maintain compatibility with their
-    # previous usage, here we virtually "inject" the variables in the
-    # "githooks" configuration section if they are undefined there and
-    # are defined in the plugin sections.
-
-    foreach my $var (qw/admin userenv/) {
-        next if exists $config->{githooks}{$var};
-        foreach my $plugin (grep {exists $config->{$_}} qw/checkacls checkjira/) {
-            if (exists $config->{$plugin}{$var}) {
-                $config->{githooks}{$var} = $config->{$plugin}{$var};
-                next;
-            }
-        }
-    }
-
-    # Up to version 0.030 each plugin had its own configuration
-    # section. From v0.031 on each plugin uses a subsection of the
-    # "githooks" section for its configuration options. In order to
-    # maintain compatibility we move the plugin's section variables to
-    # its newer subsection location. But only for the plugins that
-    # existed up to v0.030.
-
-    foreach my $section (qw/checkacls checkjira checklog checkstructure gerritchangeid/) {
-        next unless exists $config->{$section};
-        if (exists $config->{"githooks.$section"}) {
-            # If there already exists a subsection we consider this a
-            # conflict and tell the user to fix it.
-            die  __PACKAGE__, ": you have incompatible configuration sections: '$section' and 'githooks.$section'.\n",
-                "Please, rename all variables from section '$section' to the subsection 'githooks.$section'.\n";
-        } else {
-            # Otherwise, we can simply turn the section into a subsection
-            $config->{"githooks.$section"} = delete $config->{$section};
-        }
-    }
-
-    # Up to v0.031 the plugins had to be hooked explicitly to the
-    # hooks they implement by configuring the githooks.HOOK
-    # options. From v0.032 on the plugins can hook themselves to any
-    # hooks they want. The users have simply to tell which plugins
-    # they are interested in by adding them to the githooks.plugin
-    # option. Here we construct this option from the HOOK options if
-    # it's not configured yet.
-
-    unless (exists $config->{'githooks.plugin'}) {
-        foreach my $hook (grep {exists $config->{githooks}{$_}} qw/commit-msg pre-commit pre-receive post-receive update/) {
-            push @{$config->{githooks}{plugin}}, @{$config->{githooks}{$hook}};
-        }
-    }
-
-    return;
-}
-
 sub get_config {
     my ($git, $section, $var) = @_;
 
@@ -161,8 +68,6 @@ EOT
         $config{githooks}{gerrit}{enabled} //= [1];
         $config{githooks}{'abort-commit'}  //= [1];
 
-        _compatibilize_config(\%config);
-
         $git->{more}{config} = \%config;
     }
 
@@ -205,7 +110,7 @@ sub get_commit {
     my ($pipe, $ctx) = $git->command_output_pipe(
         'rev-list',
         '--no-walk',
-    # See 'git help rev-list' to understand the --pretty argument
+        # See 'git help rev-list' to understand the --pretty argument
         '--pretty=format:%H%n%T%n%P%n%aN%n%aE%n%ai%n%cN%n%cE%n%ci%n%s%n%n%b%x00',
         $commit,
     );
@@ -357,14 +262,40 @@ sub write_commit_msg_file {
     return;
 }
 
-sub get_diff_files {
-    my ($git, @args) = @_;
-    my %affected;
-    foreach ($git->command(diff => '--name-status', @args)) {
-        my ($status, $name) = split ' ', $_, 2;
-        $affected{$name} = $status;
+sub filter_files_in_index {
+    my ($git, $filter) = @_;
+    my $output = $git->command(
+        qw/diff-index --name-only --no-commit-id --cached -r -z/,
+        "--diff-filter=$filter", 'HEAD',
+    );
+    return split /\0/, $output;
+}
+
+sub filter_files_in_range {
+    my ($git, $filter, $from, $to) = @_;
+    my $output = $git->command(
+        qw/diff-tree --name-only --no-commit-id -r -z/,
+        "--diff-filter=$filter", $from, $to,
+    );
+    return split /\0/, $output;
+}
+
+sub filter_files_in_commit {
+    my ($git, $filter, $commit) = @_;
+    my $output = $git->command(
+        qw/diff-tree --name-only -m -r -z/,
+        "--diff-filter=$filter", $commit,
+    );
+    my $num_parents = 0;
+    my %files;
+    foreach my $name (split /\0/, $output) {
+        if ($name =~ /^[0-9a-f]{40}$/) {
+            ++$num_parents;
+        } else {
+            ++$files{$name};
+        }
     }
-    return \%affected;
+    return grep { $files{$_} == $num_parents } keys %files;
 }
 
 sub set_affected_ref {
@@ -536,7 +467,7 @@ Git::More - A Git extension with some goodies for hook developers.
 
 =head1 VERSION
 
-version 0.051
+version 0.052
 
 =head1 SYNOPSIS
 
@@ -549,8 +480,8 @@ version 0.051
     my $commits = $git->get_commits($oldcommit, $newcommit);
     my $message = $git->get_commit_msg('HEAD');
 
-    my $files_modified_by_commit = $git->get_diff_files('--diff-filter=AM', '--cached');
-    my $files_modified_by_push   = $git->get_diff_files('--diff-filter=AM', $oldcommit, $newcommit);
+    my $files_modified_by_commit = $git->filter_files_in_index('AM');
+    my $files_modified_by_push   = $git->filter_files_in_range('AM', $oldcommit, $newcommit);
 
 =head1 DESCRIPTION
 
@@ -773,24 +704,61 @@ defined.
 An empty line (C<\n\n>) is inserted between every pair of MSG
 arguments, if there is more than one, of course.
 
-=head2 get_diff_files DIFFARGS...
+=head2 filter_files_in_index FILTER
 
-This method invokes the command C<git diff --name-status> with extra
-options and arguments as passed to it. It returns a reference to a
-hash mapping every affected files to their affecting status. Its
-purpose is to make it easy to grok the names of files affected by a
-commit or a sequence of commits. Please, read C<git help diff> to know
-everything about its options.
+This method returns a list of the names of the files that are changed in the
+index (staging area) compared to the HEAD commit. It's useful in the
+C<pre-commit> hook when you want to know which files are being modified in
+the upcoming commit.
 
-A common usage is to grok every file added or modified in a pre-commit
-hook:
+FILTER specifies in which kind of changes you're interested in. It's passed
+as the argument to the C<--diff-filter> option of C<git diff-index>, which
+is documented like this:
 
-    my $hash_ref = $git->get_diff_files('--diff-filter=AM', '--cached');
+  --diff-filter=[(A|C|D|M|R|T|U|X|B)...[*]]
 
-Another one is to grok every file added or modified in a pre-receive
-hook:
+    Select only files that are Added (A), Copied (C), Deleted (D), Modified
+    (M), Renamed (R), have their type (i.e. regular file, symlink,
+    submodule, ...) changed (T), are Unmerged (U), are Unknown (X), or have
+    had their pairing Broken (B). Any combination of the filter characters
+    (including none) can be used. When * (All-or-none) is added to the
+    combination, all paths are selected if there is any file that matches
+    other criteria in the comparison; if there is no file that matches other
+    criteria, nothing is selected.
 
-    my $hash_ref = $git->get_diff_files('--diff-filter=AM', $old_commit, $new_commit);
+=head2 filter_files_in_range FILTER, FROM, TO
+
+This method returns a list of the names of the files that are changed
+between FROM and TO commits. It's useful in the C<update> and the
+C<pre-receive> hooks when you want to know which files are being modified in
+the commits being received by a C<git push> command.
+
+FILTER specifies in which kind of changes you're interested in. Please, read
+the C<filter_files_in_index> documetation above.
+
+FROM and TO are revision parameters (see C<git help revisions>) specifying
+two commits. They're passed as arguments to C<git diff-tree> in order to
+compare them and grok the files that differ between them.
+
+=head2 filter_files_in_commit FILTER, COMMIT
+
+This method returns a list of the names of the files that are changed in
+COMMIT. It's useful in the C<patchset-created> and the C<draft-published>
+hooks when you want to know which files are being modified in the single
+commit being received by a C<git push> command.
+
+FILTER specifies in which kind of changes you're interested in. Please, read
+the C<filter_files_in_index> documetation above.
+
+COMMIT is a revision parameter (see C<git help revisions>) specifying the
+commit. It's passed a argument to C<git diff-tree> in order to compare it to
+its parents and grok the files that changed in it.
+
+Merge commits are treated specially. Only files that are changed in COMMIT
+with respect to all of its parents are returned. The reasoning behind this
+is that if a file isn't changed with respect to one or more of COMMIT's
+parents, then it must have been checked already in those commits and we
+don't need to check it again.
 
 =head2 set_affected_ref REF OLDCOMMIT NEWCOMMIT
 
